/*
 Example guide: https://www.amebaiot.com/en/amebapro2-arduino-epaper-qrcode/

 */

#include <avr/pgmspace.h>

#include "epdif_2in9.h"
#include "epdpaint.h"
#include "image.h"
#include "qrcode.h"

// SPI Pin definition
// Any GPIO pins can be used for EPD_BUSY_Pin & EPD_RES_Pin, you may refer to the pinmap of the board you are using
#define EPD_BUSY_Pin        4
#define EPD_RES_Pin         5
#define EPD_SPI_CS_Pin      SPI_SS
#define EPD_SPI_MOSI_Pin    SPI_MOSI
#define EPD_SPI_MISO_Pin    SPI_MISO
#define EPD_SPI_CLK_Pin     SPI_SCLK

#define COLORED           0
#define UNCOLORED         1
#define QRCODE_VERSION    5

EpdIf2in9 epd2in9(EPD_BUSY_Pin, EPD_RES_Pin, EPD_SPI_CS_Pin, EPD_SPI_MOSI_Pin, EPD_SPI_MISO_Pin, EPD_SPI_CLK_Pin);
unsigned char image[1024];
Paint paint(image, 0, 0);     // width should be the multiple of 8

void setup() {
    pinMode(EPD_BUSY_Pin, INPUT);       // EPD_BUSY_Pin
    pinMode(EPD_RES_Pin, OUTPUT);       // EPD_RES_Pin
    pinMode(EPD_SPI_CS_Pin, OUTPUT);    // EPD_SPI_CS_Pin
    pinMode(EPD_SPI_MOSI_Pin, OUTPUT);  // EPD_SPI_MOSI_Pin
    pinMode(EPD_SPI_MISO_Pin, OUTPUT);  // EPD_SPI_MISO_Pin
    pinMode(EPD_SPI_CLK_Pin, OUTPUT);   // EPD_SPI_CLK_Pin
}

void loop() {
    epd2in9.EPD_SetFrameMemory_Base(gImage_basemapNA);
    PrintQRCode("https://www.amebaiot.com/en/ameba-arduino-summary/");
    epd2in9.EPD_Sleep();
}

void PrintQRCode(const char * url) {
    QRCode qrcode;
    const int ps = 2;               // pixels / square : used to define the size of QR
    uint8_t qrcodeData[qrcode_getBufferSize(QRCODE_VERSION)];
    qrcode_initText(&qrcode, qrcodeData, QRCODE_VERSION, ECC_LOW, url);

    paint.SetRotate(ROTATE_0);
    paint.SetWidth (80);
    paint.SetHeight(80);
    paint.Clear(COLOR_WHITE);

    for (uint8_t y = 0; y < qrcode.size; y++) {
        for (uint8_t x = 0; x < qrcode.size; x++) {
            if (qrcode_getModule(&qrcode, x, y)) {
                for (int xi = (x * ps + 2); xi < (x * ps + ps + 2); xi++) { // If pixel is on, we draw a "ps x ps" black square
                    for (int yi = (y * ps + 2); yi < (y * ps + ps + 2); yi++) {
                        paint.DrawPixel(xi, yi, COLOR_BLACK);
                    }
                }
            }
        }
    }
    epd2in9.EPD_ClearScreen_White();  // Display whole screen in white
    epd2in9.EPD_SetFrameMemory_Partial(30, 200, paint.GetImage(), paint.GetWidth(), paint.GetHeight());
    epd2in9.EPD_DisplayFrame();
}
